package net.wanji.web.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.dto.SignalStatusLogDTO;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.mapper.TDeviceStatusMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class SignalStatusTask {
    private final UtcFeignClients utcFeignClients;
    private final TDeviceStatusMapper tDeviceStatusMapper;
    private final ControlHistMapper controlHistMapper;

    @Scheduled(fixedRate = 1 * 60 * 1000)
    public void syncSignalStatus() {
        try {
            JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                log.error("定时任务同步信号机设备状态utcService调用异常");
                return;
            }
            List<SignalStatusLogDTO> content = (List<SignalStatusLogDTO>) jsonViewObject.getContent();
            content.forEach(signalStatusLogDTO -> {
                String signalId = signalStatusLogDTO.getSignalId();
                Integer currentSignalStatus = signalStatusLogDTO.getStatus() == null ? 0 : signalStatusLogDTO.getStatus();
                Integer currentFaultType = signalStatusLogDTO.getFaultType() == null ? 0 : signalStatusLogDTO.getFaultType();

                // 更新数据库状态
                LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                if (StringUtils.isNotEmpty(signalId)) {
                    lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, signalId);
                    TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);
                    if (isExucuteUpdate(currentSignalStatus, currentFaultType, tDeviceStatusInfo)) {
                        tDeviceStatusInfo.setStatus(currentSignalStatus);
                        tDeviceStatusInfo.setFaultType(currentFaultType);
                        tDeviceStatusMapper.updateById(tDeviceStatusInfo);
                    }
                }
                // 更新控制历史表
                String crossId = signalStatusLogDTO.getCrossId();
                Integer controlType = signalStatusLogDTO.getControlType();
                TCrossControlHist tCrossControlHist = controlHistMapper.selectRecentOne(crossId);
                if (Objects.nonNull(tCrossControlHist) && !Objects.equals(controlType, tCrossControlHist.getType())) {
                    controlHistMapper.insertOne(crossId, controlType, "未知");
                }
            });
        } catch (Exception e) {
            log.error("定时任务同步信号机设备状态远程utcService调用异常", e);
        }
    }

    private static boolean isExucuteUpdate(Integer currentSignalStatus, Integer currentFaultType, TDeviceStatusInfo tDeviceStatusInfo) {
        return Objects.nonNull(tDeviceStatusInfo) && (currentSignalStatus != tDeviceStatusInfo.getStatus() || currentFaultType != tDeviceStatusInfo.getFaultType());
    }
}
